﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.Editors;
using Infragistics.Windows;

namespace XamDataGridDifferentEditors
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public ObservableCollection<AutoTenderRegionCap> AutoTenderCaps { get; set; }
        public ObservableCollection<OriginDestinationType> OrigDestTypes { get; set; }
        public ObservableCollection<AutoTenderRegion> Regions { get; set; }

        public MainWindow()
        {
            InitializeComponent();

            Style ComboStyle = this.MainGrid.Resources["xce"] as Style;
            Style TextLStyle = this.MainGrid.Resources["xteL"] as Style;
            Style TextSStyle = this.MainGrid.Resources["xteS"] as Style;

            xamDataGrid1.FieldLayouts[0].Fields["uf"].Settings.EditorStyleSelector =
                new EditorSelector(TextSStyle, ComboStyle, TextLStyle);

            // Intialize test data
            OrigDestTypes = new ObservableCollection<OriginDestinationType>();

            OrigDestTypes.Add(new OriginDestinationType{ID = 0, Code = "REGION", Description = "Region"});
            OrigDestTypes.Add(new OriginDestinationType{ID = 1, Code = "STATE", Description = "State"});
            OrigDestTypes.Add(new OriginDestinationType{ID = 2, Code = "LOCATION", Description = "Location"});

            Regions = new ObservableCollection<AutoTenderRegion>();

            Regions.Add(new AutoTenderRegion { ID = 0, Name = "Midwest" });
            Regions.Add(new AutoTenderRegion { ID = 1, Name = "South East" });
            Regions.Add(new AutoTenderRegion { ID = 2, Name = "West Coast" });

            AutoTenderCaps = new ObservableCollection<AutoTenderRegionCap>();

            AutoTenderCaps.Add(new AutoTenderRegionCap{OriginState = "MN", DestinationState = "NY", OriginLocation = "W100",  DestinationLocation = "W500", Priority = 4,
                                                       OriginRegion = new AutoTenderRegion {ID = 0, Name = "Midwest" },
                                                       DestinationRegion = new AutoTenderRegion { ID = 1, Name = "South East" },
                                                       OriginATCapType = new OriginDestinationType { ID = 2, Code = "LOCATION", Description = "Location" },
                                                       DestinationATCapType = new OriginDestinationType { ID = 0, Code = "REGION", Description = "Region" }
            });
            AutoTenderCaps.Add(new AutoTenderRegionCap{OriginState = "AZ", DestinationState = "CA", OriginLocation = "W13300", DestinationLocation = "W50330", Priority = 23,
                                                       OriginRegion = new AutoTenderRegion { ID = 2, Name = "West Coast" },
                                                       DestinationRegion = new AutoTenderRegion { ID = 0, Name = "Midwest" },
                                                       OriginATCapType = new OriginDestinationType { ID = 1, Code = "STATE", Description = "State" },
                                                       DestinationATCapType = new OriginDestinationType { ID = 0, Code = "REGION", Description = "Region" }
            });

            // Make all of this stuff accessible in the XAML
            OuterWindow.DataContext = this;

        }

        void XCEChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            if (xamDataGrid1.ActiveRecord != null)
            {
                CellValuePresenter.FromRecordAndField(xamDataGrid1.ActiveRecord as DataRecord, (xamDataGrid1.ActiveRecord as DataRecord).Cells["uf"].Field).Editor.StartEditMode();
                (xamDataGrid1.ActiveRecord as DataRecord).Cells["uf"].EndEditMode();
                CellValuePresenter.FromRecordAndField(xamDataGrid1.ActiveRecord as DataRecord, (xamDataGrid1.ActiveRecord as DataRecord).Cells["DestinationATCapType"].Field).Editor.StartEditMode();
            }
            
        }

        private void TextEditor(object sender, Infragistics.Windows.Editors.Events.EditModeStartedEventArgs e)
        {
            (sender as XamTextEditor).Value += " ";
            (sender as XamTextEditor).Value.ToString().Remove((sender as XamTextEditor).Value.ToString().Count() - 1);
            (sender as XamTextEditor).SelectAll();
        }

        private void ComboEditorEnd(object sender, Infragistics.Windows.Editors.Events.EditModeEndingEventArgs e)
        {
            if ((sender as XamComboEditor).SelectedItem == null)
            {
                if ((sender as XamComboEditor).DisplayValue == "Midwest")
                {
                    (sender as XamComboEditor).Value = ((sender as XamComboEditor).ItemsSource as ObservableCollection<AutoTenderRegion>)[1];
                    Dispatcher.BeginInvoke(new Action(() =>
                    {
                        (sender as XamComboEditor).Value = ((sender as XamComboEditor).ItemsSource as ObservableCollection<AutoTenderRegion>)[0];
                    }), System.Windows.Threading.DispatcherPriority.Background, null);
                }
                else
                {
                    AutoTenderRegion tmp = ((sender as XamComboEditor).ItemsSource as ObservableCollection<AutoTenderRegion>).Where(t => t.Name == (sender as XamComboEditor).DisplayValue).FirstOrDefault();
                    (sender as XamComboEditor).Value = ((sender as XamComboEditor).ItemsSource as ObservableCollection<AutoTenderRegion>)[0];
                    Dispatcher.BeginInvoke(new Action(() =>
                    {
                        (sender as XamComboEditor).Value = tmp;
                    }), System.Windows.Threading.DispatcherPriority.Background, null);
                }
            }            
        }
    }

    public class AutoTenderRegionCap
    {
        public OriginDestinationType OriginATCapType { get; set; }
        public OriginDestinationType DestinationATCapType { get; set; }
        public string OriginState { get; set; }
        public string DestinationState { get; set; }
        public string OriginLocation { get; set; }
        public string DestinationLocation { get; set; }
        public AutoTenderRegion OriginRegion { get; set; }
        private AutoTenderRegion destinationRegion;

        public AutoTenderRegion DestinationRegion
        {
            get { return destinationRegion; }
            set 
            { 
                destinationRegion = value;
            }
        }
        
        public int Priority { get; set; }
    }

    public class OriginDestinationType
    {
        public int ID { get; set; }
        public string Code { get; set; }
        public string Description { get; set; }

        public override bool Equals(object obj)
        {
            if (obj is OriginDestinationType)
            {
                OriginDestinationType other = (OriginDestinationType)obj;

                if (this.ID == other.ID)
                {
                    return true;
                }
            }
            
            return false;
        }
    }

    public class AutoTenderRegion
    {

        private int id;

        public int ID
        {
            get { return id; }
            set 
            { 
                id = value;
            }
        }

        private string name;

        public string Name
        {
            get { return name; }
            set 
            { 
                name = value;
            }
        }
        

        public override bool Equals(object obj)
        {
            if (obj is AutoTenderRegion)
            {
                AutoTenderRegion other = (AutoTenderRegion)obj;

                if (this.ID == other.ID)
                {
                    return true;
                }
            }

            return false;
        }
    }
    
    public class EditorSelector : StyleSelector
    {
        private Style textLStyle;
        private Style textSStyle;
        private Style comboStyle;

        public EditorSelector(Style textSStyle, Style comboStyle, Style textLStyle)
        {
            this.textSStyle = textSStyle;
            this.comboStyle = comboStyle;
            this.textLStyle = textLStyle;
        }

        public override Style SelectStyle(object item, DependencyObject container)
        {
            if (item == null || item.ToString() != "Text" )
            {
                if (((container as CellValuePresenter).Record.DataItem as AutoTenderRegionCap).DestinationATCapType.Code == "REGION")
                {
                    return comboStyle;
                }
                else if (((container as CellValuePresenter).Record.DataItem as AutoTenderRegionCap).DestinationATCapType.Code == "STATE")
                {
                    return textSStyle;
                }
                else
                    return textLStyle;
            }
            else
                return null;
        }
    }

    public class StateConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return value;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return value;
        }
    }

    public class LocationConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return value;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return value;
        }
    }

    public class RegionConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return (value as AutoTenderRegion).Name;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value != null && targetType.IsAssignableFrom(value.GetType()))
                return value;

            if (targetType == typeof(AutoTenderRegion) && value is AutoTenderRegionCap)
            {
                return (value as AutoTenderRegionCap).DestinationRegion;
            }
            else
                return Binding.DoNothing;

            
        }
    }


}

